/***************************************************************************//**
* @file     A31R71x_hal_wt.c
* @brief    Contains all functions support for wt firmware library on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

/* Includes ----------------------------------------------------------------- */
//******************************************************************************
// Include
//******************************************************************************

#include "A31R71x_hal_wt.h"
#include "A31R71x_hal_scu.h"

/* Public Functions --------------------------------------------------------- */
//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Initialize the WT peripheral with the specified parameters.
 * @param[in]     WT_Config
 *                   Pointer to a WT_CFG_Type structure
 *                   that contains the configuration information for the specified peripheral.
 * @return        @ref HAL_Status_Type
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_Init( WT_CFG_Type* WT_Config )
{
   /* Check WT_Config */
   if( WT_Config == NULL )
   {
      return HAL_ERROR;
   }

   // enable peripheral clock
   HAL_SCU_Peripheral_EnableClock2( PPCLKEN2_WTCLKE, PPxCLKE_Enable );

   WT->CR = 0
            | ( ( WT_Config->wtClkDiv & 0x03 ) << 4 )   // WTINTV[1:0]
            | WT_CR_WTCLR
            ;
   WT->DR = ( WT_Config->wtTmrConst & 0xFFF );

   // return
   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         deinit for Watch Timer function
 * @param         None
 * @return        @ref HAL_Status_Type
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_DeInit( void )
{
   WT->CR = 0;

   // return
   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Configure the peripheral interrupt.
 * @param[in]     NewState
 *                   Next State of Interrupt Operation
 *                   -  ENABLE, DISABLE
 * @return        @ref HAL_Status_Type
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_ConfigInterrupt( FunctionalState NewState )
{
   uint32_t    reg_val = 0;

   reg_val = ( WT->CR & 0xFF );

   if( NewState == ENABLE )
   {
      reg_val |= WT_CR_WTIEN;          // WTIEN bit
   }
   else
   {
      reg_val &= ~WT_CR_WTIEN;
   }

   WT->CR = reg_val | WT_CR_WTIFLAG;   // (1<<1):  clear for WTIFLAG

   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Enable WT activity
 * @param[in]     ctrl
 *                   Next State of Functional Operation
 *                   -  ENABLE, DISABLE
 * @return        @ref HAL_Status_Type
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_Start( FunctionalState ctrl )
{
   uint32_t    tmp_reg;

   tmp_reg = WT->CR & 0xFF;
   tmp_reg &= ~( 0x1 << 7 );  // Disable watch-dog timer counter

   if( ctrl == ENABLE )
   {
      tmp_reg |= ( 0x1 << 7 );   // Enable watch timer counter
   }

   tmp_reg |= WT_CR_WTCLR;    // clear the counter and divider

   WT->CR = tmp_reg;

   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Set WT CR Register
 * @param[in]     u32WTSet
 *                      Watch Timer Control Register Setting Data
 * @return        @ref HAL_Status_Type
 * @details       This function sets the mode, external clock edge, Timer out polarity,
 *                Capture Polarity and Timer match/capture interrupt.
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_SetRegister( uint32_t u32WTSet )
{
   WT->CR = u32WTSet;      // Setting Timer 1n Control Register

   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Clear the timer status register of WT
 * @param         None
 * @return        @ref HAL_Status_Type
 *//*-------------------------------------------------------------------------*/
HAL_Status_Type HAL_WT_ClearStatus( void )
{
   WT->CR |= WT_CR_WTIFLAG;   // (1<<1):  clear for WTIFLAG

   return HAL_OK;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Get the timer status register of WT
 * @param         None
 * @return        the status register of WT
 *//*-------------------------------------------------------------------------*/
uint32_t HAL_WT_GetStatus( void )
{
   return ( WT->CR & WT_STATUS_BITMASK );
}

/*-------------------------------------------------------------------------*//**
 * @brief         Get the current value of WT
 * @param         None
 * @return        current value of WT
 *//*-------------------------------------------------------------------------*/
uint32_t HAL_WT_GetCurrentCount( void )
{
   return ( WT->CNT & 0xFFF );
}

