/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
uint16_t adcaverage( uint16_t* val );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void ADC_Configure( void );
void ADC_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "ADC_Polling Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: ADC \n\r"
                        "ADC Input Pin:     PA0(AN0) ~ PA3(AN3) \n\r"
                        "************************************************\n\r\n\r";

uint16_t    adcval[8];

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         adcaverage
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
uint16_t adcaverage( uint16_t* val )
{
   uint32_t    i;
   uint32_t    adcavg;

   adcavg = 0;
   for( i = 0; i < 8; i++ )
   {
      adcavg += val[i];
   }
   adcavg >>= 3;

   return adcavg;
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_Configure( void )
{
   ADC_CFG_Type      ADC_Config;

   // ADC configuration
   {
      // adc input pin configuration
      {
         // print
         _DBG( "PA0 ~ PA3 was configured as a AN0 ~ AN3. \n\r\n\r" );

         // configure PA0 as a AN0
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 0, 3 );

         // configure PA1 as a AN1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, 3 );

         // configure PA2 as a AN2
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 2, 3 );

         // configure PA3 as a AN3
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 3, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 3, 3 );
      }

      // adc configuration
      {
         ADC_Config.RefSel = ADC_CR_REFSEL_Vdd;
         ADC_Config.TrgSel = ADC_CR_TRIG_ADST;
         ADC_Config.InClkDiv = 14;     // 40M/14 = 2.8M (less than 3M)
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_PollingRun( void )
{
   uint32_t    i;
   uint32_t    adcavg;

   /* Infinite loop */
   while( 1 )
   {
      // adc channel 0
      HAL_ADC_ChannelSel( ADC, 0 );
      for( i = 0; i < 8; i++ )
      {
         HAL_ADC_Start( ADC );
         while( ( HAL_ADC_GetStatus( ADC ) & ADC_CR_ADCIFLAG_Msk ) != ADC_CR_ADCIFLAG_Msk ) {}
         HAL_ADC_ClearStatus( ADC );
         adcval[i] = HAL_ADC_GetData( ADC );
      }
      adcavg = adcaverage( adcval );
#ifdef _DEBUG_MSG
      _DBG( "AN0 = " );
      _DBH16( adcavg );
      _DBG( "  " );
#endif

      // adc channel 1
      HAL_ADC_ChannelSel( ADC, 1 );
      for( i = 0; i < 8; i++ )
      {
         HAL_ADC_Start( ADC );
         while( ( HAL_ADC_GetStatus( ADC ) & ADC_CR_ADCIFLAG_Msk ) != ADC_CR_ADCIFLAG_Msk ) {}
         HAL_ADC_ClearStatus( ADC );
         adcval[i] = HAL_ADC_GetData( ADC );
      }
      adcavg = adcaverage( adcval );
#ifdef _DEBUG_MSG
      _DBG( "AN1 = " );
      _DBH16( adcavg );
      _DBG( "  " );
#endif

      // adc channel 2
      HAL_ADC_ChannelSel( ADC, 2 );
      for( i = 0; i < 8; i++ )
      {
         HAL_ADC_Start( ADC );
         while( ( HAL_ADC_GetStatus( ADC ) & ADC_CR_ADCIFLAG_Msk ) != ADC_CR_ADCIFLAG_Msk ) {}
         HAL_ADC_ClearStatus( ADC );
         adcval[i] = HAL_ADC_GetData( ADC );
      }
      adcavg = adcaverage( adcval );
#ifdef _DEBUG_MSG
      _DBG( "AN2 = " );
      _DBH16( adcavg );
      _DBG( "  " );
#endif

      // adc channel 3
      HAL_ADC_ChannelSel( ADC, 3 );
      for( i = 0; i < 8; i++ )
      {
         HAL_ADC_Start( ADC );
         while( ( HAL_ADC_GetStatus( ADC ) & ADC_CR_ADCIFLAG_Msk ) != ADC_CR_ADCIFLAG_Msk ) {}
         HAL_ADC_ClearStatus( ADC );
         adcval[i] = HAL_ADC_GetData( ADC );
      }
      adcavg = adcaverage( adcval );
#ifdef _DEBUG_MSG
      _DBG( "AN3 = " );
      _DBH16( adcavg );
      _DBG( "\n\r" );
#endif

      // delay
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* ADC_Configure */
   ADC_Configure();

   /* ADC_PollingRun */
   ADC_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

