/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER20_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER20_Configure( void );
void TIMER2n_CaptureRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER2n_Capture Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:             TIMER20 \n\r"
                        "Capture Signal Generation Pin: PE8 \n\r"
                        "Timer Capture Input Pin:       PC0(T20CAP) \n\r"
                        "Prerequisite:                  Connect PE8 and PC0(T20CAP) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    fflag = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER20_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER20_IRQHandler_IT( void )
{
   uint32_t    status;

   // get interrupt flag
   status = TIMER20->CR_b.T2nCIFLAG;

   // if Timer/Counter 2n Capture Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      HAL_TIMER2n_ClearStatus( ( TIMER2n_Type* )TIMER20, TIMER2n_CR_CAPTURE_FLAG );

      // clear counter
      //    TIMER21 counter is automatically cleared after Capture interrupt is issued.
      //    So, the below function call is not necessary in TIMER21.
      HAL_TIMER2n_ClearCounter( ( TIMER2n_Type* )TIMER20 );

      // update fflag
      if( fflag == 0 )
      {
         fflag = 1;
      }
      else if( fflag == 1 )
      {
         fflag = 2;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // capture signal generation pin configurtion
   {
      // print
      _DBG( "PE8 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PE8 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 8, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 8, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER20_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER20_Configure( void )
{
   TIMER2n_CAPTURECFG_Type    TIMER2n_Config;

   // TIMER20 configuration
   {
      // timer capture input pin configuration
      {
         // print
         _DBG( "PC0 was configured as a T20CAP. \n\r\n\r" );

         // configure PC0 as a T20CAP
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF2 );
      }

      // timer capture mode configuration
      {
         TIMER2n_Config.CkSel = TIMER2n_INTERNAL_CLK;          // PCLK = 40MHz
         TIMER2n_Config.Prescaler = 40;                        // fTIMER = PCLK / 40 = 1MHz
         TIMER2n_Config.ClrMode = TIMER2n_RISING_EGDE;
         TIMER2n_Config.CAPCkSel = TIMER2n_CAP_EXTERNAL_CLK;
         TIMER2n_Config.TCLK_SEL = TIMER2n_PCLK_CLK;

         HAL_TIMER2n_Init( ( TIMER2n_Type* )TIMER20, TIMER2n_CAPTURE_MODE, &TIMER2n_Config );
         HAL_TIMER2n_ConfigInterrupt( ( TIMER2n_Type* )TIMER20, TIMER2n_CR_CAPTURE_INTR, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER20_IRQn, 3 );
      NVIC_EnableIRQ( TIMER20_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER20 );

      // enable timer (clear & start counter)
      HAL_TIMER2n_Cmd( ( TIMER2n_Type* )TIMER20, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER2n_CaptureRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER2n_CaptureRun( void )
{
   uint32_t    i;
   uint32_t    count;

   // generate capture signal
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );   // low
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 8 ) );     // high
   for( i = 0; i < 1000; i++ );                    // 1.6ms
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );   // low
   for( i = 0; i < 1000; i++ );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 8 ) );     // high

   /* Infinite loop */
   while( 1 )
   {
      switch( fflag )
      {
         case 2:
            // updae fflag
            fflag = 0;

            // get capture data
            count = HAL_TIMER2n_GetCaptureData( ( TIMER2n_Type* )TIMER20 );

            // print capture data
            _DBG( "Capture CNT = " );
            _DBD32( count );
            _DBG( "\n\r" );     // Capture data : 1400
            if( count == 1400 )
            {
               _DBG( "Capture CNT is Ok. \n\r" );
            }
            else
            {
               _DBG( "Capture CNT is not Ok. \n\r" );
            }
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER20_Configure */
   TIMER20_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER2n_CaptureRun */
   TIMER2n_CaptureRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

