/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31R71x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER20_Configure( void );
void TIMER2n_MatchPollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER2n_MatchPolling Example \n\r"
                        "  - MCU:        A31R713 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER20 \n\r"
                        "GPIO Output Pin:   PE8 ~ PE11 \n\r"
                        "Timer Output Pin:  PC0(T20OUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PE8 ~ PE11 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PE8 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 8, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 8, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );

      // configure PE9 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 9, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 9, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 9 ) );

      // configure PE10 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 10, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 10, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 10 ) );

      // configure PE11 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 11, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 11, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER20_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER20_Configure( void )
{
   TIMER2n_PERIODICCFG_Type      TIMER2n_Config;

   // TIMER20 configuration
   {
      // timer output pin configuration
      {
         // print
         _DBG( "PC0 was configured as a T20OUT. \n\r\n\r" );

         // configure PC0 as a T20OUT
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF1 );
      }

      // timer periodic mode configuration
      {
         TIMER2n_Config.CkSel = TIMER2n_INTERNAL_CLK;       // PCLK = 40MHz
         TIMER2n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
         TIMER2n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
         TIMER2n_Config.StartLevel = TIMER2n_START_LOW;
         TIMER2n_Config.TCLK_SEL = TIMER2n_PCLK_CLK;
         TIMER2n_Config.BDR = 0;                            // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
         TIMER2n_Config.CapEdge = TIMER2n_FALLING_EGDE;     // dummy parameter
         TIMER2n_Config.ExtCkEdge = TIMER2n_FALLING_EGDE;   // dummy parameter

         HAL_TIMER2n_Init( ( TIMER2n_Type* )TIMER20, TIMER2n_PERIODIC_MODE, &TIMER2n_Config );
      }
   }

   // enable timer (clear & start counter)
   HAL_TIMER2n_Cmd( ( TIMER2n_Type* )TIMER20, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER2n_MatchPollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER2n_MatchPollingRun( void )
{
   uint32_t    count = 0;
   uint8_t     alternate = 1;    // toggle flag

   /* Infinite loop */
   while( 1 )
   {
      // wait until interrupt flag is 1
      while( ( HAL_TIMER2n_GetStatus( ( TIMER2n_Type* )TIMER20 ) & TIMER2n_CR_MATCH_FLAG ) != TIMER2n_CR_MATCH_FLAG );

      // clear interrupt flag
      HAL_TIMER2n_ClearStatus( ( TIMER2n_Type* )TIMER20, TIMER2n_CR_MATCH_FLAG );

      // indicate
      if( ( count++ % 1000 ) == 0 )
      {
         // toggle PE8 ~ PE11
         if( alternate == 1 )
         {
            // clear flag
            alternate = 0;

            // output high
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 8 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 9 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 10 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 11 ) );
         }
         else
         {
            // set flag
            alternate = 1;

            // output low
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 8 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 9 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 10 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER20_Configure */
   TIMER20_Configure();

   /* TIMER2n_MatchPollingRun */
   TIMER2n_MatchPollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

